

class GamepadEmulatorAdapter:
    def __init__(self):
        pass

    def emulate_control(self, control, value):
        pass

    def emulate_multiple_controls(self, control_value_dict):
        pass
