

class GamepadReaderAdapter:
    def __init__(self):
        pass

    def get_control_state(self, control):
        pass

    def get_states_of_multiple_controls(self, controls_to_check):
        pass
